import { Application, addWeakEventListener, Observable, removeWeakEventListener, Utils, ViewBase } from '@nativescript/core';
const observableBindingHandler = {
    addChangeListener(source, target, propertyNames, changeCallback) {
        addWeakEventListener(source, Observable.propertyChangeEvent, changeCallback, target);
    },
    removeChangeListener(source, target, propertyNames, changeCallback) {
        removeWeakEventListener(source, Observable.propertyChangeEvent, changeCallback, target);
    },
    isCompatible(source) {
        return source instanceof Observable;
    },
    setValue(source, target, propertyName, propertyValue) {
        source.set(propertyName, propertyValue);
    }
};
function notifyViewBindingContextChange(args) {
    const view = args.object;
    view.notify({
        object: view,
        eventName: 'bindingContextChange',
        propertyName: 'bindingContext',
        value: view.bindingContext,
        oldValue: view.bindingContext,
    });
}
export function onBindingSourcePropertyChange(args) {
    // eslint-disable-next-line @typescript-eslint/no-this-alias
    const view = this;
    BindingHelper.startViewModelToViewUpdate(view, view.bindingContext, viewModel => {
        const $value = viewModel;
        const $parent = view.parent ? view.parent.bindingContext : null;
        if (args.propertyName in view._bindingSourceCallbackPairs) {
            view._bindingSourceCallbackPairs[args.propertyName](view, viewModel, {
                $value,
                $parent
            });
        }
    });
}
export class BindingHelper {
    _bindingHandler = observableBindingHandler;
    get dataBindingHandler() {
        return this._bindingHandler;
    }
    set dataBindingHandler(handler) {
        this._bindingHandler = handler;
    }
    static createParentsBindingInstance(view, cssTypes) {
        const instance = {};
        let parent = view.parent;
        while (parent && cssTypes.length) {
            const index = cssTypes.findIndex(cssType => cssType.toLowerCase() === parent.typeName);
            if (index >= 0) {
                const cssType = cssTypes.splice(index, 1);
                instance[cssType[0]] = parent.bindingContext;
            }
            parent = parent.parent;
        }
        // Some parents are not available soon enough, so a retry is done when view gets loaded
        if (cssTypes.length && !view.isLoaded) {
            view.off(ViewBase.loadedEvent, notifyViewBindingContextChange);
            view.once(ViewBase.loadedEvent, notifyViewBindingContextChange);
        }
        return instance;
    }
    static startViewToViewModelUpdate(view, bindingContext, callback) {
        callback(bindingContext);
    }
    static startViewModelToViewUpdate(view, bindingContext, callback) {
        const bindingResources = Application.getResources();
        const addedBindingContextProperties = [];
        let source;
        // Ensure source is an object
        if (bindingContext == null) {
            source = {};
        }
        else {
            const type = typeof bindingContext;
            if (type === 'number') {
                source = new Number(bindingContext);
            }
            else if (type === 'boolean') {
                source = new Boolean(bindingContext);
            }
            else if (type === 'string') {
                source = new String(bindingContext);
            }
            else {
                source = bindingContext;
            }
        }
        // Addition of application resources
        for (const propertyName in bindingResources) {
            if (!(propertyName in source)) {
                source[propertyName] = bindingResources[propertyName];
                addedBindingContextProperties.push(propertyName);
            }
        }
        callback(source);
        // Finally, perform a cleanup for view model
        for (const propertyName of addedBindingContextProperties) {
            delete source[propertyName];
        }
    }
    static runConverterCallback(context, converter, args, toModelDirection = false) {
        let callback;
        if (!Utils.isNullOrUndefined(converter) && Utils.isObject(converter)) {
            callback = (toModelDirection ? converter.toModel : converter.toView);
        }
        else {
            callback = converter;
        }
        return Utils.isFunction(callback) ? callback.apply(context, args) : undefined;
    }
}
//# sourceMappingURL=binding-helper.js.map